/* 


   callplaymidi.c

   Forks and calls playmidi.
   This program is as little as possible.
   It avoids forking your main program each
   time you want play a midiFile.   


   May 1996 
   Xavier HOSXE


*/

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>
#include <signal.h>
#include <sys/wait.h>
#include <time.h>

#define OK 1
#define ERROR 0


int pidPlaymidi=0;
int status;
char output[]="--";
int loop=1;


/* ------------ Usage ------------- */

void usage() {
  printf(" USAGE : \n");
  printf(" callplaymidi <card> <pipe1[0]> <pipe1[1]> <pipe2[0]> <pipe2[1]> \n");
  exit(0);
}



/* ------- Stop Playing --------- */

void stopPlaying(int sig) {
  /*  printf("callplaymidi : stopPlaying(%i) \n",sig);*/
  if (pidPlaymidi) {
    kill(pidPlaymidi,SIGKILL);
    wait(&status);
    pidPlaymidi=0;    
    loop=0;
  }
  signal(SIGUSR1,stopPlaying);
}

/* ----- No playmidi ---- */

void noPlaymidi(int sig) {
  int pidPere;
  /*printf("callPlaymidi : nooooooooooPlaymidi \n");*/
  pidPere=getppid();
  kill(pidPere,SIGUSR1);
  exit(0);
}

/* ------- fin ------------ */

void Midi_end(int sig) {
  /*  printf("callplaymidi : Midi_end() \n");*/
  stopPlaying(0);
  exit(0);
}



/* --------- Initialisation du process playmidi -------------- */

int Midi_play(char *song) {  

  if (!(pidPlaymidi=fork())) {
    int pidPere;
    /* ----- process fils ------- */
    execlp("playmidi","playmidi",output,song,0);
    pidPere=getppid();
    kill(pidPere,SIGUNUSED);
    printf("callplaymidi : playmidi is not installed... No music...\n");
    exit(1);
  }
  
  if (pidPlaymidi) {
    /*    printf("callplaymidi :  fork to %i OK\n",pidPlaymidi);*/
    return 1;
  } else {
    loop=0;
    printf("callplaymidi :  fork error  !\n");
    return 0;
  }
}


/* ----------- Main Part ------------- */


/* ================= PROGRAMME PRINCIPAL ======================= */


void main(int argc,char **argv) {
  int pipe1[2];
  int pipe2[2];
  struct stat pipeStat;
  int len;
  char msg[80];
  char un=1;
  time_t oldtime;


  signal(SIGUSR1,stopPlaying);
  signal(SIGUSR2,Midi_end);
  signal(SIGUNUSED,noPlaymidi);

  if (argc!=6) usage();

  /*  printf(" callplaymidi : Argc : %i \n",argc);
      printf(" callplaymidi : Argv : %s %s %s %s %s \n",argv[1],argv[2],argv[3],argv[4],argv[5]); 
  */

  strcpy(output,argv[1]);
  pipe1[0]   = atoi(argv[2]);
  pipe1[1]   = atoi(argv[3]);
  pipe2[0]   = atoi(argv[4]);
  pipe2[1]   = atoi(argv[5]);
  
  close(pipe1[1]);
  close(pipe2[0]);
    
  write(pipe2[1],&un,1);

  /* ========= BOUCLE PRINCIPALE =========== */
   
  while (1) {
    /*    printf("callplaymidi : waiting...\n");*/


    loop=1;

    do {
      /* -- to avoid CPU to burn -- */
      usleep(10000);
      if (fstat(pipe1[0],&pipeStat)) {
        fprintf(stderr,"callplaymidi : State Pipe error : callplaymidi...\n");
        exit(1);
      }
      len=pipeStat.st_size;
    } while (len==0);
    
    if (read(pipe1[0],msg,len) !=len) {
      fprintf(stderr,"callplaymidi : Read Pipe Error : callplaymidi \n");
      exit(2);
    }
    
    msg[len] = 0;
    /*printf("callplaymidi : recu %s \n",msg);*/

    while (loop) { 
      time(&oldtime);
      loop=1; /* loop = 0 si interuption SIGUSR1 */
      /*printf("callplaymidi : loop just before Midi_play\n");*/
      Midi_play(msg);
      wait(&status);
      /*      printf("callplaymidi : time : %i oldtime %i\n",time(NULL),oldtime);*/
      if ((time(NULL)-oldtime)<2) {
	loop =0;
	printf("callplaymidi : Music shorter than 2 seconds (problem ?)\n");
      }
    }
  } 
}



