/* -----------------------------------

   Joystick.cc
   To allow all part of the programm
   to acced Joystick  (or keyboard) !!
   
   Xavier HOSXE 
   January 1996  
   
   ----------------------------------- */


#include <stdio.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <unistd.h>
#include <rawkey.h>
#include <vga.h>
#include <vgakeyboard.h>

#include "Joystick.h"
#include "SpriteElem.h"
#include "ListSprite.h"

extern ListSprite *list;
extern short joystick;
extern short fin;
extern void Pause();
extern short enPause;
extern short enJeu;
short keyb;
short joyDev[2];
struct calib joyCal[2];

/* -------------- Sauve ecran raw -------------------- */

void Sauve() {
#ifdef SAVE_RAW
  int i,j,k;
  char ck;
  int r,g,b;
  char rc,gc,bc;
  char nom[20];
  static int n=1;
  FILE *fichierGif;

  sprintf(nom,"%i.raw",n++);
  if ((fichierGif=fopen(nom,"w"))==NULL) {
    printf("Erreur enregistrement image %i\n",n);
    return;
  }

  for (j=0;j<480;j++)
    for (i=0;i<640;i++) {
      k = gl_getpixel(i,j+pageFlipping[page]);
      ck = (char) k;
      fwrite(&ck,1,1,fichierGif);
    }

  fclose(fichierGif);
#endif
}



/* ------------- Joystick Exit ----------------------- */

void exitJoystick() {
  if (keyb) {
    printf("Rawmode exit...\n");
    rawmode_exit();
  } 
}

/* ------------- Keybooard init ------------------------- */

void initKeyboard() {
  if (!rawmode_init()) {
    fprintf(stdout," RawKey ERROR ! \n");
    printf("No joystick, No rawkey mode...\n");
    printf("No SpaceBoom... Bye.\n");
    exit(1);
  }
  keyb=1;
}


/* ------------ Joystick init + calibrate ----------------- */

int initJoystick(short w) {
  char deviceChar[20];
  struct JS_DATA_TYPE *js;
  short status;
  short ok = 0;
    

  keyb=0;

  js = new JS_DATA_TYPE;

  if ((w<0) || (w>1)) return 0; 
  sprintf(deviceChar,"/dev/js%i",w);

  usleep(200000);
  if ((joyDev[w]=open(deviceChar,O_RDONLY))==-1) {
    printf("Cannot open joystick %i device...\n",w);
    return 0;
  } 
  joyCal[w].x.min = 10000;
  joyCal[w].x.max = -10000;
  joyCal[w].y.min = 10000;
  joyCal[w].y.max = -10000;

  printf("Calibrate joystick %i\n",w);

  while (!ok) {
    printf("Make a circle with your joystick, center it,then press button when done...\n");
    
    status=read(joyDev[w],js,JS_RETURN);
    if (status != JS_RETURN) {
      printf("Cannot read joystick %i device...\n",w);
      return 0;
    }
    usleep(200000);
    
    while (!(js->buttons & 1)) {
      if (js->x < joyCal[w].x.min)
	joyCal[w].x.min = js->x;
      if (js->x > joyCal[w].x.max)
	joyCal[w].x.max = js->x;
      if (js->y < joyCal[w].y.min)
	joyCal[w].y.min = js->y;
      if (js->y > joyCal[w].y.max)
	joyCal[w].y.max = js->y;  
      read(joyDev[w],js,JS_RETURN);
    }
    joyCal[w].x.med = js->x;
    joyCal[w].y.med = js->y;
    if ((joyCal[w].x.min<joyCal[w].x.med) && (joyCal[w].x.med<joyCal[w].x.max) && (joyCal[w].y.min<joyCal[w].y.med) && (joyCal[w].y.med<joyCal[w].y.max))
      ok = 1;
    if (!ok)
      printf("Bad joystick calibration !\n");
  }
  return 1;
}

struct movement *readJoystick(short w) {
  static struct movement ret;

  ret.right = 0;
  ret.left = 0;
  ret.up =0;
  ret.down = 0;
    
  if (keyb) {
    ret.b1=0;
    ret.b2=0;

    scan_keyboard();
    
    if (is_key_pressed(CURSOR_LEFT))
      ret.left=1;
    if (is_key_pressed(CURSOR_RIGHT))
      ret.right=1;
    if (is_key_pressed(CURSOR_DOWN))
      ret.down=1;
    if (is_key_pressed(CURSOR_UP))
      ret.up=1;
    if (is_key_pressed(LEFT_CTRL) || is_key_pressed(ENTER_KEY))
      ret.b1=1;
    if (is_key_pressed(LEFT_ALT))
      ret.b2=1;
    if (is_key_pressed(ESCAPE_KEY) || (is_key_pressed(SCANCODE_Q))) 
      if ((!enPause) && (enJeu)) {
	((SpriteElem *)(list->mfirst))->Die();
      }
    if (is_key_pressed(SCANCODE_P))
      if ((!enPause) && (enJeu)) {
	Pause();
      }
#ifdef SAVE_RAW
    if (is_key_pressed(SCANCODE_S))
      Sauve();
#endif

  } else {
    
    static JS_DATA_TYPE js;
    static key;
  
    read(joyDev[w],&js,JS_RETURN);
    
    if (js.x>((joyCal[w].x.med+joyCal[w].x.max)>>1)) {
      ret.right = 1;
    } else if (js.x<((joyCal[w].x.med+joyCal[w].x.min)>>1)) {
      ret.left = 1;
    }
    if (js.y>((joyCal[w].y.med+joyCal[w].y.max)>>1)) {
      ret.down = 1;
    } else if (js.y<((joyCal[w].y.med+joyCal[w].y.min)>>1)) {
      ret.up = 1;
    }
    ret.b1 = (js.buttons & 1);
    ret.b2 = (js.buttons & 2);   
    key=vga_getkey();
    switch(key) {
    case 'p':
    case 'P':
      if ((!enPause) && (enJeu)) {
	Pause();
      }
      break;
    case 'q':
    case 'Q':
      if ((!enPause)  && (enJeu)) {
	((SpriteElem *)(list->mfirst))->Die();
      }
      break;
    }  
  }  
  return &ret;
  
}
